function isEmptyNode(node) {
    if (!(node === null || node === void 0 ? void 0 : node.content) || node.content.length === 0)
        return true;
    return false;
}
export const typstStrings = {
    ',': (state) => (state.data.inFunction ? 'comma' : ','),
    '&': (state) => (state.data.inArray ? ',' : '&'),
    '/': '\\/',
    ';': '\\;',
    '~': 'med',
};
function splitStrings(node) {
    var _a, _b, _c;
    if (((_b = (_a = node.args) === null || _a === void 0 ? void 0 : _a[0].content) === null || _b === void 0 ? void 0 : _b.length) === 1 &&
        ((_c = node.args) === null || _c === void 0 ? void 0 : _c[0].content)[0].type === 'string') {
        node.args[0].content = node.args[0].content[0].content
            .split('')
            .map((l) => ({ type: 'string', content: l }));
    }
}
export const typstMacros = {
    cdot: 'dot.op',
    to: 'arrow.r',
    rightarrow: 'arrow.r',
    leftarrow: 'arrow.l',
    leftrightarrow: 'arrow.l.r',
    gets: 'arrow.l',
    infin: 'infinity',
    infty: 'infinity',
    nonumber: '',
    int: 'integral',
    iint: 'integral.double',
    sqrt: (state, node) => {
        var _a;
        if (isEmptyNode((_a = node.args) === null || _a === void 0 ? void 0 : _a[0]))
            return 'sqrt';
        return 'root';
    },
    vec: 'arrow',
    mathbf: 'bold',
    boldsymbol: 'bold',
    mathrm: 'upright',
    textrm: 'upright',
    rm: 'upright',
    mathcal: 'cal',
    mathfrak: 'frak',
    pm: 'plus.minus',
    partial: 'diff',
    _: (state, node) => {
        splitStrings(node);
        return '_';
    },
    '^': (state, node) => {
        splitStrings(node);
        return '^';
    },
    left: (state, node) => {
        var _a;
        const args = node.args;
        node.args = [];
        const left = ((_a = args === null || args === void 0 ? void 0 : args[0].content) === null || _a === void 0 ? void 0 : _a[0]).content;
        if (left === '(')
            return '(';
        if (left === '[')
            return '[';
        if (left === '{')
            return '{';
        if (left === '|')
            return '|';
        if (left === '.')
            return '';
        throw new Error(`Undefined left bracket: ${left}`);
    },
    right: (state, node) => {
        var _a;
        const args = node.args;
        node.args = [];
        const right = ((_a = args === null || args === void 0 ? void 0 : args[0].content) === null || _a === void 0 ? void 0 : _a[0]).content;
        if (right === ')')
            return ')';
        if (right === ']')
            return ']';
        if (right === '}')
            return '}';
        if (right === '|')
            return '|';
        if (right === '.')
            return '';
        throw new Error(`Undefined right bracket: ${right}`);
    },
    operatorname: (state, node) => {
        var _a;
        const text = (_a = node.args) === null || _a === void 0 ? void 0 : _a.slice(-1)[0];
        node.args = [{ type: 'macro', content: 'text', args: [text] }];
        return 'op';
    },
    '\\': (state, node) => {
        node.args = [];
        if (state.data.inArray)
            return ';';
        return '\\\n';
    },
    sim: 'tilde',
    cong: 'tilde.equiv',
    simeq: 'tilde.eq',
    ne: '!=',
    phi: 'phi.alt',
    varepsilon: 'epsilon',
    propto: 'prop',
    doteq: 'dot(eq)',
    ge: 'gt.eq',
    geq: 'gt.eq',
    le: 'lt.eq',
    leq: 'lt.eq',
    neq: 'eq.not',
    otimes: 'times.circle',
    dot: 'dot',
    ddot: 'dot.double',
    dots: 'dots.h',
    ldots: 'dots.h',
    vdots: 'dots.v',
    ddots: 'dots.down',
    subseteq: 'subset.eq',
    cdots: 'dots.h.c',
    cap: 'sect',
    cup: 'union',
    widehat: 'hat',
    // Spaces
    ',': 'thin',
    ':': 'med',
    ';': 'thick',
    '!': '#h(-1em)',
    quad: 'quad',
    qquad: 'wide',
    prod: 'product',
    biggl: '',
    biggr: '',
    mathbb: (state, node) => {
        var _a, _b, _c, _d, _e;
        const text = (_e = (_d = (_c = (_b = (_a = node.args) === null || _a === void 0 ? void 0 : _a.slice(-1)[0]) === null || _b === void 0 ? void 0 : _b.content) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.content) !== null && _e !== void 0 ? _e : '';
        const letters = text
            .split('')
            .map((l) => `${l}${l}`)
            .join(' ');
        node.args = [];
        return letters;
    },
    mathscr: (state) => {
        state.useMacro(`#let scr(it) = text(features: ("ss01",), box($cal(it)$))`);
        return 'scr';
    },
    overset: (state, node) => {
        var _a;
        state.useMacro('#import "@preview/ouset:0.2.0": *');
        node.args = (_a = node.args) === null || _a === void 0 ? void 0 : _a.reverse();
        return 'overset';
    },
    underset: (state, node) => {
        var _a;
        state.useMacro('#import "@preview/ouset:0.2.0": *');
        node.args = (_a = node.args) === null || _a === void 0 ? void 0 : _a.reverse();
        return 'underset';
    },
};
export const typstEnvs = {
    array: (state, node) => {
        state.data.inArray = true;
        state.openFunction('mat');
        // TODO: transform the surrounding brackets into arguments
        state.write('delim: #none,');
        state.writeChildren(node);
        state.closeFunction();
        state.data.inArray = false;
    },
};
